# -*- coding: utf-8 -*-

def get_article_templates(article, user):
    return (
        ('standard.html', 'Standard'),
        ('homepage.html', 'Homepage'),
        ('blog.html', 'Blog'),
        ('standard_en.html', 'English'),
    )