{% load i18n %}
<li>
    <h4><a href="{{ a.get_absolute_url }}">{{ item.title|safe }}</a></h4>
    <p>{{ item.content|safe|truncatewords_html:150 }}</p>
    <br/>
    <div class="article_details">
        <span class="date">
            Posté il y a {% blocktrans with item.created|timesince as when %}{{ when }}
            {% endblocktrans %}</span>
    </div>
</li>