{% load i18n %}
{% with node.content_object as content %}
<script>
{% if node.in_navigation %}
$("#navnode_in_navigation").html('{% trans "Hide node in navigation" %}');
{% else %}
$("#navnode_in_navigation").html('{% trans "Show node in navigation" %}');
{% endif %}
</script>
<h3>{% block navtree_content_title %}{{node.content_type|capfirst}}: {{content}}{% endblock %}</h3>
{% block navtree_content_command%}
<ul class="content-links">
    <li><a href="{{admin_url}}">{% trans "Edit in admin"%}</a></li>
    <li><a href="{{content.get_absolute_url}}">{% trans "View on site"%}</a></li>
    {% block navtree_content_extracommand%}
    {% endblock %}
</ul>
{% endblock %}
{% endwith %}